"""
Program options: 

    -d datadir   Specify data directory
    -g glob      Specify selected file names (a la Unix)
    -f first     First file to operate on. (default: 0, mean
                 the first of the list)
    -n number    How many file to consider. (default: all
                 selected files.
    -s           Step mode (Wait after each file)
"""

import os, json, glob, sys, getopt
from astropy.io import fits
import numpy as np

class Pipeline:
    def __init__(self, operate, common_args=None):
        ddir, glbn, first, number, step = get_args()
        self.filenames = glob.glob(os.path.join(ddir, glbn))
        self.filenames.sort()
        self.step = step
        if common_args is None:
            self.common_args = {}
        else:
            self.common_args = common_args
        if first:
            self.filenames = self.filenames[first:]
        if number:
            self.filenames = self.filenames[:number]
        self.operate = operate

    def _getdata(self, filename):
        raise Exception("You must implement _getdata()!")

    def run(self):
        for step, filename in enumerate(self.filenames):
            img, args = self._getdata(filename)
            res = self.operate(img, step, args, self.common_args)
            if self.step:
                input("*** Press enter to go on")

def get_args():
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'd:g:f:n:sh')
    except getopt.GetoptError:
        print("Argument error (-h: help)")
        sys.exit()

    ddir = "."
    glbn = "*.*"
    first = None
    number = None
    step = False
    for (o, v) in opts:
        if o == '-h':
            print(__doc__)
            sys.exit()
        elif o == '-d':
            ddir = v
        elif o == "-g":
            glbn = v
        elif o == "-f":
            first = int(v)
        elif o == "-n":
            number = int(v)
        elif o == "-s":
            step = True
    return (ddir, glbn, first, number, step)

class RawPipeline(Pipeline):
    "Read raw data files"
    def _getdata(self, filename):
        data, hdr = fits.getdata(filename, 0, header=True)
        return data, {"hdr": hdr, "filename": filename}

class CalPipeline(Pipeline):
    "Read calibration files"
    def _getdata(self, filename):
        data = np.load(filename)
        return data, {"filename": filename}
